@@ -107,16 +107,16 @@ EditTagDialog::EditTagDialog(Application *app, QWidget *parent)
 
   cover_options_.default_output_image_ = AlbumCoverLoader::ScaleAndPad(cover_options_, QImage(":/pictures/cdcase.png"));
 
-  connect(app_->album_cover_loader(), SIGNAL(ImageLoaded(quint64,QImage,QImage)), SLOT(ArtLoaded(quint64,QImage,QImage)));
+  connect(app_->album_cover_loader(), SIGNAL(ImageLoaded(quint64, QUrl, QImage, QImage)), SLOT(AlbumCoverLoaded(quint64, QUrl, QImage, QImage)));
 
 #if defined(HAVE_GSTREAMER) && defined(HAVE_CHROMAPRINT)
   connect(tag_fetcher_, SIGNAL(ResultAvailable(Song, SongList)), results_dialog_, SLOT(FetchTagFinished(Song, SongList)), Qt::QueuedConnection);
-  connect(tag_fetcher_, SIGNAL(Progress(Song,QString)), results_dialog_, SLOT(FetchTagProgress(Song,QString)));
+  connect(tag_fetcher_, SIGNAL(Progress(Song, QString)), results_dialog_, SLOT(FetchTagProgress(Song, QString)));
   connect(results_dialog_, SIGNAL(SongChosen(Song, Song)), SLOT(FetchTagSongChosen(Song, Song)));
   connect(results_dialog_, SIGNAL(finished(int)), tag_fetcher_, SLOT(Cancel()));
 #endif
 
-  album_cover_choice_controller_->SetApplication(app_);
+  album_cover_choice_controller_->Init(app_);
 
   ui_->setupUi(this);
   ui_->splitter->setSizes(QList<int>() << 200 << width() - 200);
@@ -337,6 +337,7 @@ QVariant EditTagDialog::Data::value(const Song &song, const QString &id) {
   if (id == "grouping") return song.grouping();
   if (id == "genre") return song.genre();
   if (id == "comment") return song.comment();
+  if (id == "lyrics") return song.lyrics();
   if (id == "track") return song.track();
   if (id == "disc") return song.disc();
   if (id == "year") return song.year();
@@ -356,6 +357,7 @@ void EditTagDialog::Data::set_value(const QString &id, const QVariant &value) {
   else if (id == "grouping") current_.set_grouping(value.toString());
   else if (id == "genre") current_.set_genre(value.toString());
   else if (id == "comment") current_.set_comment(value.toString());
+  else if (id == "lyrics") current_.set_lyrics(value.toString());
   else if (id == "track") current_.set_track(value.toInt());
   else if (id == "disc") current_.set_disc(value.toInt());
   else if (id == "year") current_.set_year(value.toInt());
@@ -505,13 +507,13 @@ void EditTagDialog::UpdateSummaryTab(const Song &song) {
     art_is_set = false;
   }
   else if (!song.art_manual().isEmpty()) {
-    summary += tr("Cover art set from %1").arg(song.art_manual()).toHtmlEscaped();
+    summary += tr("Cover art set from %1").arg(song.art_manual().toString()).toHtmlEscaped();
   }
   else if (song.has_embedded_cover()) {
     summary += tr("Cover art from embedded image");
   }
   else if (!song.art_automatic().isEmpty()) {
-    summary += tr("Cover art loaded automatically from %1").arg(song.art_automatic()).toHtmlEscaped();
+    summary += tr("Cover art loaded automatically from %1").arg(song.art_automatic().toString()).toHtmlEscaped();
   }
   else {
     summary += tr("Cover art not set").toHtmlEscaped();
@@ -541,7 +543,7 @@ void EditTagDialog::UpdateSummaryTab(const Song &song) {
 
   ui_->filetype->setText(song.TextForFiletype());
 
-  if (song.url().scheme() == "file")
+  if (song.url().isLocalFile())
     ui_->filename->setText(QDir::toNativeSeparators(song.url().toLocalFile()));
   else
     ui_->filename->setText(song.url().toString());
@@ -559,7 +561,9 @@ void EditTagDialog::UpdateStatisticsTab(const Song &song) {
 
 }
 
-void EditTagDialog::ArtLoaded(quint64 id, const QImage &scaled, const QImage &original) {
+void EditTagDialog::AlbumCoverLoaded(const quint64 id, const QUrl &cover_url, const QImage &scaled, const QImage &original) {
+
+  Q_UNUSED(cover_url);
 
   if (id == cover_art_id_) {
     ui_->art->setPixmap(QPixmap::fromImage(scaled));
@@ -623,9 +627,9 @@ void EditTagDialog::LoadCoverFromFile() {
 
   const QModelIndexList sel = ui_->song_list->selectionModel()->selectedIndexes();
 
-  QString cover = album_cover_choice_controller_->LoadCoverFromFile(song);
+  QUrl cover_url = album_cover_choice_controller_->LoadCoverFromFile(song);
 
-  if (!cover.isEmpty()) UpdateCoverOf(*song, sel, cover);
+  if (!cover_url.isEmpty()) UpdateCoverOf(*song, sel, cover_url);
 
 }
 
@@ -645,9 +649,9 @@ void EditTagDialog::LoadCoverFromURL() {
 
   const QModelIndexList sel = ui_->song_list->selectionModel()->selectedIndexes();
 
-  QString cover = album_cover_choice_controller_->LoadCoverFromURL(song);
+  QUrl cover_url = album_cover_choice_controller_->LoadCoverFromURL(song);
 
-  if (!cover.isEmpty()) UpdateCoverOf(*song, sel, cover);
+  if (!cover_url.isEmpty()) UpdateCoverOf(*song, sel, cover_url);
 }
 
 void EditTagDialog::SearchForCover() {
@@ -657,9 +661,9 @@ void EditTagDialog::SearchForCover() {
 
   const QModelIndexList sel = ui_->song_list->selectionModel()->selectedIndexes();
 
-  QString cover = album_cover_choice_controller_->SearchForCover(song);
+  QUrl cover_url = album_cover_choice_controller_->SearchForCover(song);
 
-  if (!cover.isEmpty()) UpdateCoverOf(*song, sel, cover);
+  if (!cover_url.isEmpty()) UpdateCoverOf(*song, sel, cover_url);
 }
 
 void EditTagDialog::UnsetCover() {
@@ -669,8 +673,8 @@ void EditTagDialog::UnsetCover() {
 
   const QModelIndexList sel = ui_->song_list->selectionModel()->selectedIndexes();
 
-  QString cover = album_cover_choice_controller_->UnsetCover(song);
-  UpdateCoverOf(*song, sel, cover);
+  QUrl cover_url = album_cover_choice_controller_->UnsetCover(song);
+  UpdateCoverOf(*song, sel, cover_url);
 }
 
 void EditTagDialog::ShowCover() {
@@ -683,7 +687,7 @@ void EditTagDialog::ShowCover() {
   album_cover_choice_controller_->ShowCover(*song);
 }
 
-void EditTagDialog::UpdateCoverOf(const Song &selected, const QModelIndexList &sel, const QString &cover) {
+void EditTagDialog::UpdateCoverOf(const Song &selected, const QModelIndexList &sel, const QUrl &cover_url) {
 
   if (!selected.is_valid() || selected.id() == -1) return;
 
@@ -696,7 +700,7 @@ void EditTagDialog::UpdateCoverOf(const Song &selected, const QModelIndexList &s
 
     Song *other_song = &data_[i].original_;
     if (selected.artist() == other_song->artist() && selected.album() == other_song->album()) {
-      other_song->set_art_manual(cover);
+      other_song->set_art_manual(cover_url);
     }
   }
 
@@ -779,9 +783,9 @@ bool EditTagDialog::eventFilter(QObject *o, QEvent *e) {
         const QModelIndexList sel = ui_->song_list->selectionModel()->selectedIndexes();
         Song *song = GetFirstSelected();
 
-        const QString cover = album_cover_choice_controller_->SaveCover(song, event);
-        if (!cover.isEmpty()) {
-          UpdateCoverOf(*song, sel, cover);
+        const QUrl cover_url = album_cover_choice_controller_->SaveCover(song, event);
+        if (!cover_url.isEmpty()) {
+          UpdateCoverOf(*song, sel, cover_url);
         }
 
         break;
@@ -892,9 +896,7 @@ void EditTagDialog::FetchTagSongChosen(const Song &original_song, const Song &ne
 }
 #endif
 
-void EditTagDialog::SongSaveComplete(TagReaderReply *reply, const QString filename, const Song song) {
-
-  reply->deleteLater();
+void EditTagDialog::SongSaveComplete(TagReaderReply *reply, const QString &filename, const Song &song) {
 
   pending_--;
 
@@ -910,4 +912,6 @@ void EditTagDialog::SongSaveComplete(TagReaderReply *reply, const QString filena
 
   if (pending_ <= 0) AcceptFinished();
 
+  reply->deleteLater();
+
 }
