@@ -21,30 +21,24 @@
 
 #include "config.h"
 
-#include <functional>
-#include <algorithm>
-
 #include <QtGlobal>
 #include <QWidget>
 #include <QTimer>
 #include <QList>
 #include <QString>
-#include <QStringList>
 #include <QPixmap>
 #include <QPalette>
 #include <QColor>
 #include <QFont>
-#include <QMenu>
-#include <QSortFilterProxyModel>
 #include <QStandardItem>
-#include <QSettings>
+#include <QMenu>
 #include <QAction>
+#include <QSettings>
 #include <QtEvents>
 
 #include "core/application.h"
 #include "core/logging.h"
 #include "core/mimedata.h"
-#include "core/timeconstants.h"
 #include "core/iconloader.h"
 #include "internet/internetsongmimedata.h"
 #include "collection/collectionfilterwidget.h"
@@ -64,18 +58,16 @@ using std::swap;
 
 const int InternetSearchView::kSwapModelsTimeoutMsec = 250;
 
-InternetSearchView::InternetSearchView(Application *app, InternetSearch *engine, QString settings_group, SettingsDialog::Page settings_page, QWidget *parent)
+InternetSearchView::InternetSearchView(QWidget *parent)
     : QWidget(parent),
-      app_(app),
-      engine_(engine),
-      settings_group_(settings_group),
-      settings_page_(settings_page),
+      app_(nullptr),
+      engine_(nullptr),
       ui_(new Ui_InternetSearchView),
       context_menu_(nullptr),
       last_search_id_(0),
-      front_model_(new InternetSearchModel(engine_, this)),
-      back_model_(new InternetSearchModel(engine_, this)),
-      current_model_(front_model_),
+      front_model_(nullptr),
+      back_model_(nullptr),
+      current_model_(nullptr),
       front_proxy_(new InternetSearchSortModel(this)),
       back_proxy_(new InternetSearchSortModel(this)),
       current_proxy_(front_proxy_),
@@ -84,24 +76,15 @@ InternetSearchView::InternetSearchView(Application *app, InternetSearch *engine,
       {
 
   ui_->setupUi(this);
+
   ui_->progressbar->hide();
   ui_->progressbar->reset();
 
-  front_model_->set_proxy(front_proxy_);
-  back_model_->set_proxy(back_proxy_);
-
   ui_->search->installEventFilter(this);
   ui_->results_stack->installEventFilter(this);
 
   ui_->settings->setIcon(IconLoader::Load("configure"));
 
-  // Must be a queued connection to ensure the InternetSearch handles it first.
-  connect(app_, SIGNAL(SettingsChanged()), SLOT(ReloadSettings()), Qt::QueuedConnection);
-
-  connect(ui_->search, SIGNAL(textChanged(QString)), SLOT(TextEdited(QString)));
-  connect(ui_->results, SIGNAL(AddToPlaylistSignal(QMimeData*)), SIGNAL(AddToPlaylist(QMimeData*)));
-  connect(ui_->results, SIGNAL(FocusOnFilterSignal(QKeyEvent*)), SLOT(FocusOnFilter(QKeyEvent*)));
-
   // Set the appearance of the results list
   ui_->results->setItemDelegate(new InternetSearchItemDelegate(this));
   ui_->results->setAttribute(Qt::WA_MacShowFocusRect, false);
@@ -123,6 +106,35 @@ InternetSearchView::InternetSearchView(Application *app, InternetSearch *engine,
   help_font.setBold(true);
   ui_->label_helptext->setFont(help_font);
 
+}
+
+InternetSearchView::~InternetSearchView() { delete ui_; }
+
+void InternetSearchView::Init(Application *app, InternetSearch *engine, const QString &settings_group, const SettingsDialog::Page settings_page, const bool artists, const bool albums, const bool songs) {
+
+  app_ = app;
+  engine_ = engine;
+  settings_group_ = settings_group;
+  settings_page_ = settings_page;
+  artists_ = artists;
+  albums_ = albums;
+  songs_ = songs;
+
+  front_model_ = new InternetSearchModel(engine_, this);
+  back_model_ = new InternetSearchModel(engine_, this);
+
+  front_proxy_ = new InternetSearchSortModel(this);
+  back_proxy_ = new InternetSearchSortModel(this);
+
+  front_model_->set_proxy(front_proxy_);
+  back_model_->set_proxy(back_proxy_);
+
+  current_model_ = front_model_;
+  current_proxy_ = front_proxy_;
+
+  // Must be a queued connection to ensure the InternetSearch handles it first.
+  connect(app_, SIGNAL(SettingsChanged()), SLOT(ReloadSettings()), Qt::QueuedConnection);
+  
   // Set up the sorting proxy model
   front_proxy_->setSourceModel(front_model_);
   front_proxy_->setDynamicSortFilter(true);
@@ -132,10 +144,6 @@ InternetSearchView::InternetSearchView(Application *app, InternetSearch *engine,
   back_proxy_->setDynamicSortFilter(true);
   back_proxy_->sort(0);
 
-  swap_models_timer_->setSingleShot(true);
-  swap_models_timer_->setInterval(kSwapModelsTimeoutMsec);
-  connect(swap_models_timer_, SIGNAL(timeout()), SLOT(SwapModels()));
-
   // Add actions to the settings menu
   group_by_actions_ = CollectionFilterWidget::CreateGroupByActions(this);
   QMenu *settings_menu = new QMenu(this);
@@ -144,28 +152,33 @@ InternetSearchView::InternetSearchView(Application *app, InternetSearch *engine,
   settings_menu->addAction(IconLoader::Load("configure"), tr("Configure %1...").arg(Song::TextForSource(engine->source())), this, SLOT(OpenSettingsDialog()));
   ui_->settings->setMenu(settings_menu);
 
+  swap_models_timer_->setSingleShot(true);
+  swap_models_timer_->setInterval(kSwapModelsTimeoutMsec);
+  connect(swap_models_timer_, SIGNAL(timeout()), SLOT(SwapModels()));
+
   connect(ui_->radiobutton_search_artists, SIGNAL(clicked(bool)), SLOT(SearchArtistsClicked(bool)));
   connect(ui_->radiobutton_search_albums, SIGNAL(clicked(bool)), SLOT(SearchAlbumsClicked(bool)));
   connect(ui_->radiobutton_search_songs, SIGNAL(clicked(bool)), SLOT(SearchSongsClicked(bool)));
-
   connect(group_by_actions_, SIGNAL(triggered(QAction*)), SLOT(GroupByClicked(QAction*)));
 
+  connect(ui_->search, SIGNAL(textChanged(QString)), SLOT(TextEdited(QString)));
+  connect(ui_->results, SIGNAL(AddToPlaylistSignal(QMimeData*)), SIGNAL(AddToPlaylist(QMimeData*)));
+  connect(ui_->results, SIGNAL(FocusOnFilterSignal(QKeyEvent*)), SLOT(FocusOnFilter(QKeyEvent*)));
+
   // These have to be queued connections because they may get emitted before our call to Search() (or whatever) returns and we add the ID to the map.
 
-  connect(engine_, SIGNAL(UpdateStatus(QString)), SLOT(UpdateStatus(QString)));
-  connect(engine_, SIGNAL(ProgressSetMaximum(int)), SLOT(ProgressSetMaximum(int)), Qt::QueuedConnection);
-  connect(engine_, SIGNAL(UpdateProgress(int)), SLOT(UpdateProgress(int)), Qt::QueuedConnection);
+  connect(engine_, SIGNAL(UpdateStatus(const int, const QString&)), SLOT(UpdateStatus(const int, const QString&)));
+  connect(engine_, SIGNAL(ProgressSetMaximum(const int, const int)), SLOT(ProgressSetMaximum(const int, const int)), Qt::QueuedConnection);
+  connect(engine_, SIGNAL(UpdateProgress(const int, const int)), SLOT(UpdateProgress(const int, const int)), Qt::QueuedConnection);
 
-  connect(engine_, SIGNAL(AddResults(int, InternetSearch::ResultList)), SLOT(AddResults(int, InternetSearch::ResultList)), Qt::QueuedConnection);
-  connect(engine_, SIGNAL(SearchError(int, QString)), SLOT(SearchError(int, QString)), Qt::QueuedConnection);
-  connect(engine_, SIGNAL(ArtLoaded(int, QPixmap)), SLOT(ArtLoaded(int, QPixmap)), Qt::QueuedConnection);
+  connect(engine_, SIGNAL(AddResults(const int, InternetSearch::ResultList)), SLOT(AddResults(const int, const InternetSearch::ResultList)), Qt::QueuedConnection);
+  connect(engine_, SIGNAL(SearchError(const int, const QString&)), SLOT(SearchError(const int, const QString&)), Qt::QueuedConnection);
+  connect(engine_, SIGNAL(AlbumCoverLoaded(const int, const QPixmap&)), SLOT(AlbumCoverLoaded(const int, const QPixmap&)), Qt::QueuedConnection);
 
   ReloadSettings();
 
 }
 
-InternetSearchView::~InternetSearchView() { delete ui_; }
-
 void InternetSearchView::ReloadSettings() {
 
   QSettings s;
@@ -176,11 +189,9 @@ void InternetSearchView::ReloadSettings() {
   const bool pretty = s.value("pretty_covers", true).toBool();
   front_model_->set_use_pretty_covers(pretty);
   back_model_->set_use_pretty_covers(pretty);
-  s.endGroup();
 
   // Internet search settings
 
-  s.beginGroup(settings_group_);
   search_type_ = InternetSearch::SearchType(s.value("type", int(InternetSearch::SearchType_Artists)).toInt());
   switch (search_type_) {
     case InternetSearch::SearchType_Artists:
@@ -195,9 +206,9 @@ void InternetSearchView::ReloadSettings() {
   }
 
   SetGroupBy(CollectionModel::Grouping(
-      CollectionModel::GroupBy(s.value("group_by1", int(CollectionModel::GroupBy_AlbumArtist)).toInt()),
-      CollectionModel::GroupBy(s.value("group_by2", int(CollectionModel::GroupBy_Album)).toInt()),
-      CollectionModel::GroupBy(s.value("group_by3", int(CollectionModel::GroupBy_None)).toInt())));
+      CollectionModel::GroupBy(s.value("search_group_by1", int(CollectionModel::GroupBy_AlbumArtist)).toInt()),
+      CollectionModel::GroupBy(s.value("search_group_by2", int(CollectionModel::GroupBy_Album)).toInt()),
+      CollectionModel::GroupBy(s.value("search_group_by3", int(CollectionModel::GroupBy_None)).toInt())));
   s.endGroup();
 
 }
@@ -242,22 +253,27 @@ void InternetSearchView::TextEdited(const QString &text) {
 
 }
 
-void InternetSearchView::AddResults(int id, const InternetSearch::ResultList &results) {
+void InternetSearchView::AddResults(const int id, const InternetSearch::ResultList &results) {
+
   if (id != last_search_id_) return;
   if (results.isEmpty()) return;
   ui_->label_status->clear();
   ui_->progressbar->reset();
   ui_->progressbar->hide();
   current_model_->AddResults(results);
+
 }
 
-void InternetSearchView::SearchError(const int id, const QString error) {
+void InternetSearchView::SearchError(const int id, const QString &error) {
+
+  if (id != last_search_id_) return;
   error_ = true;
   ui_->label_helptext->setText(error);
   ui_->label_status->clear();
   ui_->progressbar->reset();
   ui_->progressbar->hide();
   ui_->results_stack->setCurrentWidget(ui_->help_page);
+
 }
 
 void InternetSearchView::SwapModels() {
@@ -278,7 +294,7 @@ void InternetSearchView::SwapModels() {
 
 }
 
-void InternetSearchView::LazyLoadArt(const QModelIndex &proxy_index) {
+void InternetSearchView::LazyLoadAlbumCover(const QModelIndex &proxy_index) {
 
   if (!proxy_index.isValid() || proxy_index.model() != front_proxy_) {
     return;
@@ -316,12 +332,12 @@ void InternetSearchView::LazyLoadArt(const QModelIndex &proxy_index) {
   const InternetSearch::Result result = item->data(InternetSearchModel::Role_Result).value<InternetSearch::Result>();
 
   // Load the art.
-  int id = engine_->LoadArtAsync(result);
+  int id = engine_->LoadAlbumCoverAsync(result);
   art_requests_[id] = source_index;
 
 }
 
-void InternetSearchView::ArtLoaded(int id, const QPixmap &pixmap) {
+void InternetSearchView::AlbumCoverLoaded(const int id, const QPixmap &pixmap) {
 
   if (!art_requests_.contains(id)) return;
   QModelIndex index = art_requests_.take(id);
@@ -423,6 +439,16 @@ bool InternetSearchView::ResultsContextMenuEvent(QContextMenuEvent *event) {
 
   context_menu_->addSeparator();
 
+  if (artists_ || albums_ || songs_) {
+    if (artists_)
+      context_actions_ << context_menu_->addAction(IconLoader::Load("folder-new"), tr("Add to artists"), this, SLOT(AddArtists()));
+    if (albums_)
+      context_actions_ << context_menu_->addAction(IconLoader::Load("folder-new"), tr("Add to albums"), this, SLOT(AddAlbums()));
+    if (songs_)
+      context_actions_ << context_menu_->addAction(IconLoader::Load("folder-new"), tr("Add to songs"), this, SLOT(AddSongs()));
+    context_menu_->addSeparator();
+  }
+
   if (ui_->results->selectionModel() && ui_->results->selectionModel()->selectedRows().length() == 1) {
     context_actions_ << context_menu_->addAction(IconLoader::Load("search"), tr("Search for this"), this, SLOT(SearchForThis()));
   }
@@ -519,7 +545,7 @@ void InternetSearchView::SetGroupBy(const CollectionModel::Grouping &g) {
 
   // Clear requests: changing "group by" on the models will cause all the items to be removed/added again,
   // so all the QModelIndex here will become invalid. New requests will be created for those
-  // songs when they will be displayed again anyway (when InternetSearchItemDelegate::paint will call LazyLoadArt)
+  // songs when they will be displayed again anyway (when InternetSearchItemDelegate::paint will call LazyLoadAlbumCover)
   art_requests_.clear();
   // Update the models
   front_model_->SetGroupBy(g, true);
@@ -528,9 +554,9 @@ void InternetSearchView::SetGroupBy(const CollectionModel::Grouping &g) {
   // Save the setting
   QSettings s;
   s.beginGroup(settings_group_);
-  s.setValue("group_by1", int(g.first));
-  s.setValue("group_by2", int(g.second));
-  s.setValue("group_by3", int(g.third));
+  s.setValue("search_group_by1", int(g.first));
+  s.setValue("search_group_by2", int(g.second));
+  s.setValue("search_group_by3", int(g.third));
   s.endGroup();
 
   // Make sure the correct action is checked.
@@ -549,18 +575,21 @@ void InternetSearchView::SetGroupBy(const CollectionModel::Grouping &g) {
 }
 
 void InternetSearchView::SearchArtistsClicked(bool checked) {
+  Q_UNUSED(checked);
   SetSearchType(InternetSearch::SearchType_Artists);
 }
 
 void InternetSearchView::SearchAlbumsClicked(bool checked) {
+  Q_UNUSED(checked);
   SetSearchType(InternetSearch::SearchType_Albums);
 }
 
 void InternetSearchView::SearchSongsClicked(bool checked) {
+  Q_UNUSED(checked);
   SetSearchType(InternetSearch::SearchType_Songs);
 }
 
-void InternetSearchView::SetSearchType(InternetSearch::SearchType type) {
+void InternetSearchView::SetSearchType(const InternetSearch::SearchType type) {
   search_type_ = type;
   QSettings s;
   s.beginGroup(settings_group_);
@@ -569,15 +598,54 @@ void InternetSearchView::SetSearchType(InternetSearch::SearchType type) {
   TextEdited(ui_->search->text());
 }
 
-void InternetSearchView::UpdateStatus(QString text) {
+void InternetSearchView::UpdateStatus(const int id, const QString &text) {
+
+  if (id != last_search_id_) return;
   ui_->progressbar->show();
   ui_->label_status->setText(text);
+
 }
 
-void InternetSearchView::ProgressSetMaximum(int max) {
+void InternetSearchView::ProgressSetMaximum(const int id, const int max) {
+
+  if (id != last_search_id_) return;
   ui_->progressbar->setMaximum(max);
+
 }
 
-void InternetSearchView::UpdateProgress(int progress) {
+void InternetSearchView::UpdateProgress(const int id, const int progress) {
+
+  if (id != last_search_id_) return;
   ui_->progressbar->setValue(progress);
+
+}
+
+void InternetSearchView::AddArtists() {
+
+  MimeData *data = SelectedMimeData();
+  if (!data) return;
+  if (const InternetSongMimeData *internet_song_data = qobject_cast<const InternetSongMimeData*>(data)) {
+    emit AddArtistsSignal(internet_song_data->songs);
+  }
+
+}
+
+void InternetSearchView::AddAlbums() {
+
+  MimeData *data = SelectedMimeData();
+  if (!data) return;
+  if (const InternetSongMimeData *internet_song_data = qobject_cast<const InternetSongMimeData*>(data)) {
+    emit AddAlbumsSignal(internet_song_data->songs);
+  }
+
+}
+
+void InternetSearchView::AddSongs() {
+
+  MimeData *data = SelectedMimeData();
+  if (!data) return;
+  if (const InternetSongMimeData *internet_song_data = qobject_cast<const InternetSongMimeData*>(data)) {
+    emit AddSongsSignal(internet_song_data->songs);
+  }
+
 }
